from argparse import ArgumentParser
from k_hop_dominating_set_gurobi import __version__
from k_hop_dominating_set_gurobi import k_hop_dom_set
from k_hop_dominating_set_gurobi import lp_to_nx_graph
import networkx as nx

def cli(args=None):
    p = ArgumentParser(
        description="A set of python scripts to solve k hop dominating set variants using ILP and gurobi as MIP solver ",
        conflict_handler='resolve'
    )
    p.add_argument(
        '-V', '--version',
        action='version',
        help='Show the conda-prefix-replacement version number and exit.',
        version="k_hop_dominating_set_gurobi %s" % __version__,
    )
    p.add_argument(
        '-k', '--k',
        help='parameter k of the k hop dominating set',
        type=int
    )
    p.add_argument(
        '-g', '--graph',
        help="input graph as .graphml file")
    p.add_argument(
        '-mtz', 
        action='store_true',
        help='use Miller-Tucker-Zemlin constraints to ensure connectivity (default is a lazy approach using vertex separators)')
    p.add_argument(
        '-imn', 
        action='store_true',
        help='use intermediate node constraint to shrink space of feasible solutions. Might exclude optimal solutions.')
    p.add_argument(
        '-rpl', 
        action='store_true',
        help='use constraint to reduce path length from the root to each node by the size of the solution to shrink space of feasible solutions.)')
    p.add_argument(
        '-gaus', 
        action='store_true',
        help='use constraint to reduce path length from the root to each node by a sum using the gaussian sum formula.')
    p.add_argument(
        '-pre', 
        action='store_true',
        help='preadd vertex separator constraints.')
    
    
    
    args = p.parse_args(args)
    filename = args.graph

    if filename.endswith(".lp"):
        G = lp_to_nx_graph.read(args.graph)
    elif filename.endswith(".graphml"):
        G = nx.read_graphml(args.graph)
    elif filename.endswith(".gml"):
        G = nx.read_gml(args.graph)
    else:
        return 1

    additionalConstraints = []
        
    if args.imn:
        additionalConstraints.append(k_hop_dom_set.AdditionalConstraint.INTERMEDIATE_NODE_CONSTRAINT)
    if args.rpl:
        additionalConstraints.append(k_hop_dom_set.AdditionalConstraint.SIMPLE_PATH_LENGHT_CONSTRAINT)
    if args.gaus: 
        additionalConstraints.append(k_hop_dom_set.AdditionalConstraint.GAUSSIAN_SUM_FORMULA)
    if args.pre:
        additionalConstraints.append(k_hop_dom_set.AdditionalConstraint.PREADD)

    if args.mtz:
        ds = k_hop_dom_set.RootedConnectecKHopDominatingSet(G, args.k, constraints = k_hop_dom_set.ConnectivityConstraint.MILLER_TUCKER_ZEMLIN, additionalConstraints = additionalConstraints)
    else:
        ds = k_hop_dom_set.RootedConnectecKHopDominatingSet(G, args.k, additionalConstraints = additionalConstraints)


    
    ds.solve_and_draw()

    # No return value means no error.
    # Return a value of 1 or higher to signify an error.
    # See https://docs.python.org/3/library/sys.html#sys.exit


if __name__ == '__main__':
    import sys
    cli(sys.argv[1:])
